package net.lucamasini.testcase.server.boundary;

import lombok.extern.slf4j.Slf4j;
import net.lucamasini.testcase.server.boundary.resource.HasDelete;
import net.lucamasini.testcase.server.boundary.resource.HasFindByTypedId;
import net.lucamasini.testcase.server.producers.BarUtente;
import net.lucamasini.testcase.shared.common.ResponseMessages;
import net.lucamasini.testcase.shared.entity.Bar;
import net.lucamasini.testcase.shared.entity.BarLayout;
import org.jboss.resteasy.annotations.cache.NoCache;

import javax.enterprise.context.Dependent;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceContext;
import javax.transaction.Transactional;
import javax.validation.Valid;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static javax.ws.rs.core.Response.Status.CONFLICT;

@Slf4j
@Dependent
@Path("/layout")
public class BarLayoutService implements HasFindByTypedId<BarLayout, String>, HasDelete<BarLayout, String> {

    @PersistenceContext
    EntityManager em;

    @Inject
    @BarUtente
    Bar barUtente;

    @Override
    public EntityManager getEm() {
        return em;
    }

    @Override
    public Class<BarLayout> getEntityType() {
        return BarLayout.class;
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("bar/{barId}")
    @NoCache
    public BarLayout getBarLayout(@PathParam(value = "barId") String barId) {
        BarLayout barLayout = HasFindByTypedId.super.findByTypeId(barId);
        return barLayout;
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @Transactional
    public BarLayout save(@Valid BarLayout layout)   {
        return saveOrUpdate(layout);
    }



    BarLayout saveOrUpdate(BarLayout entity) {

        BarLayout retVal = entity;

        if (em.find(BarLayout.class,entity.getBar().getId())==null) {
            em.persist(entity);
        } else {
            try {
                retVal = em.merge(entity);
                em.flush();
            } catch (OptimisticLockException e) {
                throw new WebApplicationException(ResponseMessages.OPT_LOCK, CONFLICT);
            }
        }

        return retVal;
    }

    @DELETE
    @Path("bar/{barId}")
    @Transactional
    public Response deleteBarLayout(@PathParam(value = "barId") String barId) {
        HasDelete.super.delete(barId);
        return Response.ok().build();
    }



}


