package net.lucamasini.testcase.server.boundary.resource;

import net.lucamasini.testcase.shared.common.ResponseMessages;

import javax.persistence.EntityManager;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import java.sql.SQLIntegrityConstraintViolationException;


public interface HasDelete<TEntity, TId> extends IsResource<TEntity> {
    default void delete(TId id) {
        EntityManager em = getEm();
        TEntity entity = em.find(getEntityType(), id);
        if (entity != null) {
            try {
                em.remove(entity);
                em.flush();
            } catch (Exception e) {
                if (e.getCause() != null && e.getCause().getCause() != null && e.getCause().getCause() instanceof SQLIntegrityConstraintViolationException) {
                    throw new WebApplicationException(String.format(ResponseMessages.ENTITY_NOT_REMOVEABLE, getEntityType(), id), Response.Status.CONFLICT);
                } else {
                    throw e;
                }
            }
        }
    }
}
