package net.lucamasini.testcase.server.boundary.resource;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import net.lucamasini.testcase.shared.dto.PaginatedSearchResultWithLastModified;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.*;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import static net.lucamasini.testcase.shared.common.SharedConstants.*;

public interface HasFindByFilters<T> extends IsResource<T> {

    @Deprecated
    default Response findByFilters(UriInfo uriInfo, Integer start, Integer limit, Date ifModifiedSince) {

        Multimap<String, String> filters = transformQueryParamsToFilterMap(uriInfo.getQueryParameters());

        Multimap<String, String> filtersForLastModified = transformQueryParamsToFilterMap(uriInfo.getQueryParameters());
        filtersForLastModified.removeAll(FILTER_STATO_PIATTO);
        filtersForLastModified.removeAll(FILTER_STATO_ORDINE);
        filtersForLastModified.removeAll(FILTER_STATO_ORDINE_CHIOSCO);
        filtersForLastModified.removeAll(FILTER_STATO_ORDINE_SERVITO);

        Date lastModifiedTS = new Date(0);

        if (ifModifiedSince == null || lastModifiedTS.after(ifModifiedSince)) {

            Date today = asDayStart(new Date());

            TypedQuery<T> typedQuery = getTypedQuery(filters, today);
            if (start != null && limit != null) {
                typedQuery.setFirstResult(start).setMaxResults(limit);
            }
            final List<T> items = typedQuery.getResultList();
            final int count = (start != null && limit != null) ? count(filters, today) : items.size();

            return Response.ok(new PaginatedSearchResultWithLastModified<>(count, items, lastModifiedTS)).lastModified(lastModifiedTS).build();
        }
        return Response.notModified().build();

    }

    default Response findByFilters(UriInfo uriInfo, Integer start, Integer limit) {

        Multimap<String, String> filters = ArrayListMultimap.create();
        if (uriInfo != null) {
            filters.putAll(transformQueryParamsToFilterMap(uriInfo.getQueryParameters()));
        }


        TypedQuery<T> typedQuery = getTypedQuery(filters, null);
        if (start != null && limit != null) {
            typedQuery.setFirstResult(start);
            if(limit>0)
               typedQuery.setMaxResults(limit);
        }
        final List<T> items = typedQuery.getResultList();
        final int count = (start != null && limit != null) ? count(filters, null) : items.size();

        return Response.ok(new PaginatedSearchResultWithLastModified<>(count, items, null)).build();
    }


    default List<T> findByFilters(Multimap<String, String> filters) {
        return getTypedQuery(filters, null).getResultList();
    }

    default List<T> findByFiltersForToday(Multimap<String, String> filters) {
        Date today = asDayStart(new Date());
        TypedQuery<T> typedQuery = getTypedQuery(filters, today);
        List<T> items = typedQuery.getResultList();
        return items;
    }

    default Multimap<String, String> transformQueryParamsToFilterMap(MultivaluedMap<String, String> queryParams) {

        Multimap<String, String> filters = ArrayListMultimap.create();
        for (String key : queryParams.keySet()) {
            filters.putAll(key, queryParams.get(key));
        }

        return filters;
    }

    default TypedQuery<T> getTypedQuery(Multimap<String, String> filters, @Deprecated Date ifModifiedSince) {
        EntityManager em = getEm();
        Class<T> entityType = getEntityType();
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<T> query = criteriaBuilder.createQuery(entityType);
        Root<T> root = query.from(entityType);
        addJoins(root);
        query.where(getPredicates(filters, root, ifModifiedSince));
        query.orderBy(orderBy(root));
        query.select(root).distinct(true);
        return em.createQuery(query);
    }


    default int count(Multimap<String, String> filters, @Deprecated Date ifModifiedSince) {
        EntityManager em = getEm();
        CriteriaBuilder criteriaBuilder = em.getCriteriaBuilder();
        CriteriaQuery<Long> query = criteriaBuilder.createQuery(Long.class);
        Root<T> root = query.from(getEntityType());
        query.where(getPredicates(filters, root, ifModifiedSince));
        query.select(criteriaBuilder.count(root));
        return em.createQuery(query).getSingleResult().intValue();
    }

    default Predicate[] getPredicates(Multimap<String, String> filters, Root<T> root, @Deprecated Date ifModifiedSince) {
        List<Predicate> predicates = buildPredicates(filters, root, ifModifiedSince);
        return predicates.toArray(new Predicate[predicates.size()]);
    }

    List<Predicate> buildPredicates(Multimap<String, String> filters, Root<T> root, @Deprecated Date ifModifiedSince);

    void addJoins(Root<T> root);

    List<Order> orderBy(Root<T> root);

    default Date asDayStart(Date date) {
        return setTimeOfDay(date, 0, 0, 1);
    }

    default Date setTimeOfDay(Date date, int hour, int minute, int second) {
        Date dateCopy = new Date(date.getTime());
        Calendar cal = Calendar.getInstance();
        cal.setTime(dateCopy);
        cal.set(Calendar.HOUR_OF_DAY, hour);
        cal.set(Calendar.MINUTE, minute);
        cal.set(Calendar.SECOND, second);
        return cal.getTime();
    }
}
