package net.lucamasini.testcase.server.boundary.resource;

import net.lucamasini.testcase.shared.common.ResponseMessages;

import javax.persistence.EntityManager;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceUnitUtil;
import javax.ws.rs.WebApplicationException;

import static javax.ws.rs.core.Response.Status.CONFLICT;

public interface HasSaveOrUpdate<T> extends IsResource<T> {

    default T saveOrUpdateInternal(T entity) {

        EntityManager em = getEm();
        PersistenceUnitUtil persistenceUnitUtil = em.getEntityManagerFactory().getPersistenceUnitUtil();
        Object id = persistenceUnitUtil.getIdentifier(entity);

        if (id == null) {
            em.persist(entity);
        } else {
            try {
                entity = em.merge(entity);
                em.flush();
            } catch (OptimisticLockException e) {
                throw new WebApplicationException(ResponseMessages.OPT_LOCK, e, CONFLICT);
            }
        }

        return entity;
    }

}

