package net.lucamasini.testcase.shared.dto;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import net.lucamasini.testcase.shared.entity.BarLayout;
import net.lucamasini.testcase.shared.entity.Ordine;
import net.lucamasini.testcase.shared.entity.PiattoOrdinato;
import org.jboss.errai.common.client.api.annotations.Portable;

import java.util.*;

import static net.lucamasini.testcase.shared.dto.Device.*;

@Getter
@Portable
public enum StatoPiattoOrdinato implements HasTransition<StatoPiattoOrdinato> {

    APERTO {
        @Override
        public List<StatoPiattoOrdinato> getProssimiStati() {
            return Arrays.asList(StatoPiattoOrdinato.INCARICO, StatoPiattoOrdinato.PRONTO, StatoPiattoOrdinato.EVASO, StatoPiattoOrdinato.CANCELLATO, StatoPiattoOrdinato.SCARTATO);
        }

        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.publish(POSTAZIONE_FRONT);
        }
    },
    INCARICO {
        @Override
        public List<StatoPiattoOrdinato> getProssimiStati() {
            return Arrays.asList(StatoPiattoOrdinato.PRONTO, StatoPiattoOrdinato.EVASO);
        }

        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.actions().inCarico(context);
            context.publish(MONITOR_VASSOIO, POSTAZIONE_BACK);
        }
    },
    PRONTO {
        @Override
        public List<StatoPiattoOrdinato> getProssimiStati() {
            return Arrays.asList(StatoPiattoOrdinato.CANCELLATO, StatoPiattoOrdinato.EVASO);
        }

        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.actions().pronto(context);
            context.publish(MONITOR_VASSOIO, POSTAZIONE_BACK, POSTAZIONE_FRONT);
        }
    },
    EVASO {
        @Override
        public List<StatoPiattoOrdinato> getProssimiStati() {
            return Arrays.asList(StatoPiattoOrdinato.CANCELLATO);
        }

        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.actions().evaso(context);
            context.publish(MONITOR_VASSOIO, POSTAZIONE_BACK, POSTAZIONE_FRONT);
        }
    },
    CANCELLATO {
        @Override
        public List<StatoPiattoOrdinato> getProssimiStati() {
            return Arrays.asList(StatoPiattoOrdinato.APERTO);
        }

        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.actions().cancellato(context);
            context.publish(MONITOR_VASSOIO, POSTAZIONE_BACK, POSTAZIONE_FRONT);
        }
    },
    SCARTATO {
        @Override
        public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
            context.actions().scartato(context);
            context.publish(MONITOR_VASSOIO, POSTAZIONE_BACK);
        }
    },
    ;

    @AllArgsConstructor
    @NoArgsConstructor
    @Getter
    public static class PiattoOrdinatoContestoStato extends ContestoStato {
        private PiattoOrdinato piattoOrdinato;

        public PiattoOrdinatoContestoStato(PiattoOrdinato piattoOrdinato, BarLayout barLayout) {
            super.barLayout = barLayout;
            this.piattoOrdinato = piattoOrdinato;
        }

        public ActionStati actions() {
            return (super.getBarLayout()).getFlusso().getManager(piattoOrdinato.getOrdine().getCanale());
        }

        @Override
        public Ordine getOrdine() {
            return piattoOrdinato.getOrdine();
        }
    }

    public static List<StatoPiattoOrdinato> stringsToStati(Collection<String> strings) {
        List<StatoPiattoOrdinato> stati = new ArrayList<>();
        for (String s : strings) {
            stati.add(StatoPiattoOrdinato.valueOf(s));
        }
        return stati;
    }

    public final boolean cambiaStato(PiattoOrdinatoContestoStato context) {
        boolean statoModificato = false;

        PiattoOrdinato piattoOrdinato = context.getPiattoOrdinato();

        if (piattoOrdinato.getStato() != this) {

            if (this != APERTO && !piattoOrdinato.getStato().getProssimiStati().contains(this)) {
                throw new RuntimeException("for test purpose only");
            }

            this.preCambiaStato(context);
            piattoOrdinato.setStato(this);
            statoModificato = true;
            this.postCambiaStato(context);

        }
        return statoModificato;
    }

    public void preCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
    }

    public void postCambiaStato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
    }

    @Override
    public List<StatoPiattoOrdinato> getProssimiStati() {
        return Collections.emptyList();
    }

    @Override
    public boolean isStatoFinale() {
        return getProssimiStati().isEmpty();
    }
}
