package net.lucamasini.testcase.shared.dto.manager;

import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import net.lucamasini.testcase.shared.dto.ActionStati;
import net.lucamasini.testcase.shared.dto.StatoOrdine;
import net.lucamasini.testcase.shared.dto.StatoPiattoOrdinato;
import net.lucamasini.testcase.shared.entity.Abbinamento;
import net.lucamasini.testcase.shared.entity.Piatto;
import net.lucamasini.testcase.shared.entity.PiattoOrdinato;

import java.util.Arrays;
import java.util.Date;

@Slf4j
public abstract class AbstractLayoutManager implements ActionStati {

    @Override
    public boolean isBevandeInPreparazioneAllowed(StatoOrdine.OrdineContestoStato context) {
        return context.getOrdine().getPiattiOrdinati().stream()
                .filter(p -> Arrays.asList(Piatto.Preparazione.CUCINA, Piatto.Preparazione.VASSOIO).contains(p.getPiatto().getPreparazione()))
                .allMatch(p -> StatoPiattoOrdinato.PRONTO == p.getStato());
    }

    @Override
    public boolean isCancellazioneAllowed(StatoOrdine.OrdineContestoStato context) {
        return !Arrays.asList(StatoOrdine.BEVANDE_IN_PREPARAZIONE, StatoOrdine.PIATTI_IN_PREPARAZIONE).contains(context.getOrdine().getStato());
    }

    @Override
    public void cancellato(StatoOrdine.OrdineContestoStato context) {
        cancellaPiattiOrdinati(context);
    }

    private void cancellaPiattiOrdinati(StatoOrdine.OrdineContestoStato context) {
        context.getOrdine().getPiattiOrdinati().forEach(p ->
                StatoPiattoOrdinato.CANCELLATO.cambiaStato(
                        new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(p, context.getBarLayout())));
    }

    @Override
    public void nonRitirato(StatoOrdine.OrdineContestoStato context) {
        cancellaPiattiOrdinati(context);
    }

    @Override
    public void cancellato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
        context.getPiattoOrdinato().cancellaAbbinamenti();
    }

    @Override
    public final void inConsegna(StatoOrdine.OrdineContestoStato context) {
        postInConsegna(context);
    }

    @Override
    public final void evaso(StatoOrdine.OrdineContestoStato context) {
        postEvaso(context);
    }

    @Override
    public final void evaso(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
        context.getPiattoOrdinato().evadiAbbinamenti();
    }

    @Override
    public final void pronto(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
        PiattoOrdinato piatto = context.getPiattoOrdinato();
        piatto.setQuantitaEvasa(piatto.getQuantita());
        if (piatto.hasAbbinamenti() && Piatto.Preparazione.A_VISTA == piatto.getPiatto().getPreparazione()) {
            for (Abbinamento abbinamento : piatto.getAbbinamenti()) {
                abbinamento.setStato(Abbinamento.Stato.PRONTO);
            }
        }
        postPronto(context);
    }

    @Override
    public final void pronto(StatoOrdine.OrdineContestoStato context) {
        context.getOrdine().setDataPronto(new Date());
        postPronto(context);
    }


    @Override
    public final void bevandeInPreparazione(StatoOrdine.OrdineContestoStato context) {
        postBevandeInPreparazione(context);
    }

    @Override
    public final void chiuso(StatoOrdine.OrdineContestoStato context) {

        context.getOrdine().getPiattiOrdinati().stream()
                .filter(p -> !p.getPiatto().getPreparazione().equals(Piatto.Preparazione.A_VISTA) && Strings.isNullOrEmpty(p.getPiatto().getSottoPreparazione()))
                .forEach(p -> p.getPiatto().setSottoPreparazione("PIATTI"));
        if (context.getOrdine().getDataChiusura() == null) {
            context.getOrdine().setDataChiusura(new Date());
        }
        postChiuso(context);
    }

    @Override
    public void scartato(StatoOrdine.OrdineContestoStato context) {
        context.getOrdine().getPiattiOrdinati().stream()
                .filter(p -> p.getStato() != StatoPiattoOrdinato.CANCELLATO)
                .forEach(p -> StatoPiattoOrdinato.SCARTATO.cambiaStato(
                        new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(p, context.getBarLayout())));
    }

    @Override
    public void scartato(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
        context.getPiattoOrdinato().scartaAbbinamenti();
    }

    protected void postEvaso(StatoOrdine.OrdineContestoStato context) {
    }

    protected void postChiuso(StatoOrdine.OrdineContestoStato context) {
    }

    protected void postPiattiInPreparazione(StatoOrdine.OrdineContestoStato context) {
    }

    protected void postBevandeInPreparazione(StatoOrdine.OrdineContestoStato context) {
    }

    protected void postInConsegna(StatoOrdine.OrdineContestoStato context) {
    }

    protected void postPronto(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {

    }

    protected void postPronto(StatoOrdine.OrdineContestoStato context) {

    }
}
