package net.lucamasini.testcase.shared.dto.manager;

import net.lucamasini.testcase.shared.dto.Device;
import net.lucamasini.testcase.shared.dto.StatoOrdine;
import net.lucamasini.testcase.shared.dto.StatoPiattoOrdinato;
import net.lucamasini.testcase.shared.entity.Ordine;
import net.lucamasini.testcase.shared.entity.Piatto;
import net.lucamasini.testcase.shared.entity.PiattoOrdinato;

import java.util.Date;

public class AppManager extends BackManager {

    @Override
    public boolean isCancellazioneAllowed(StatoOrdine.OrdineContestoStato context) {
        return true;
    }

    @Override
    public StatoOrdine getProssimoStatoDopoBevande(StatoOrdine.OrdineContestoStato context) {

        Ordine ordine = context.getOrdine();
        if (ordine.isFreddo() && context.isBeforeFineFasciaFreddi(ordine.getDataInserimento()) && ordine.getDataCheckin() == null) {
            return StatoOrdine.PRONTO;
        }
        return StatoOrdine.IN_CONSEGNA;
    }

    @Override
    public void sospeso(StatoOrdine.OrdineContestoStato context) {
        context.publish(Device.MONITOR_VASSOIO);
    }

    @Override
    public void postPronto(StatoOrdine.OrdineContestoStato context) {
        Ordine ordine = context.getOrdine();
        for (PiattoOrdinato po : ordine.getPiattiOrdinati()) {
            if (po.isForPreparazione(Piatto.Preparazione.A_VISTA) && po.isEvadibile()) {
                StatoPiattoOrdinato.EVASO.cambiaStato(new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(po, context.getBarLayout()));
            }
        }
    }
    @Override
    public void postInConsegna(StatoOrdine.OrdineContestoStato context) {
        super.postInConsegna(context);
        context.publish(Device.MONITOR_VASSOIO);
    }

    @Override
    protected void postChiuso(StatoOrdine.OrdineContestoStato context) {
        context.getOrdine().setDataChiusura(new Date());
    }

    @Override
    protected void postPiattiInPreparazione(StatoOrdine.OrdineContestoStato context) {
        context.getOrdine().setDataPresaInCarico(new Date());
    }
}
