package net.lucamasini.testcase.shared.dto.manager;

import net.lucamasini.testcase.shared.dto.Device;
import net.lucamasini.testcase.shared.dto.StatoOrdine;
import net.lucamasini.testcase.shared.dto.StatoPiattoOrdinato;
import net.lucamasini.testcase.shared.entity.Ordine;
import net.lucamasini.testcase.shared.entity.Piatto;
import net.lucamasini.testcase.shared.entity.PiattoOrdinato;

import java.util.Date;

public abstract class BackManager extends AbstractLayoutManager {

    @Override
    public void postBevandeInPreparazione(StatoOrdine.OrdineContestoStato context) {
        Ordine ordine = context.getOrdine();
        for (PiattoOrdinato po : ordine.getPiattiOrdinati()) {

            if (po.isForPreparazione(Piatto.Preparazione.CUCINA, Piatto.Preparazione.VASSOIO) && po.isEvadibile()) {
                StatoPiattoOrdinato.EVASO.cambiaStato(new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(po, context.getBarLayout()));
            }
        }

        if (!ordine.hasPiattiForPreparazione(Piatto.Preparazione.A_VISTA) || !context.getBarLayout().isHasPdaBevande()) {
            getProssimoStatoDopoBevande(context).cambiaStato(context);
        }

        if( context.getBarLayout().isHasPdaBevande() && !context.getBarLayout().isHasMonitorVassoio() ) {
            context.publish(Device.MONITOR_VASSOIO, Device.POSTAZIONE_BACK, Device.NUMERI);
        }
    }

    @Override
    public void postInConsegna(StatoOrdine.OrdineContestoStato context) {

        Ordine ordine = context.getOrdine();
        for (PiattoOrdinato po : ordine.getPiattiOrdinati()) {
            if (po.isForPreparazione(Piatto.Preparazione.A_VISTA) && po.isEvadibile()) {
                StatoPiattoOrdinato.EVASO.cambiaStato(new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(po, context.getBarLayout()));
            }
        }

        if (!context.getBarLayout().isHasPdaBevande()) {
            context.publish(Device.MONITOR_VASSOIO);
        }
        context.publish(Device.POSTAZIONE_FORZATURA, Device.NUMERI, Device.NOTIFICA_IN_CONSEGNA);

    }

    public StatoOrdine getProssimoStatoDopoBevande(StatoOrdine.OrdineContestoStato context) {
        return StatoOrdine.IN_CONSEGNA;
    }

    @Override
    public final void piattiInPreparazione(StatoOrdine.OrdineContestoStato context) {
        Ordine ordine = context.getOrdine();

        if ( ordine.getDataPresaInCarico() == null) { //TODO e se lo riprendiamo in carico che data bisogna mettere
            ordine.setDataPresaInCarico(new Date());
        }
        if (!context.getBarLayout().isHasPdaBevande()) {
            for (PiattoOrdinato po : ordine.getPiattiOrdinati()) {
                if (po.isForPreparazione(Piatto.Preparazione.A_VISTA)) {
                    StatoPiattoOrdinato.INCARICO.cambiaStato(new StatoPiattoOrdinato.PiattoOrdinatoContestoStato(po,
                            context.getBarLayout()));
                }
            }
        }
    }

    @Override
    public void inCarico(StatoPiattoOrdinato.PiattoOrdinatoContestoStato context) {
        if (!context.getBarLayout().isHasPdaBevande() && context.getPiattoOrdinato().isForPreparazione(Piatto.Preparazione.A_VISTA)) {
                    StatoPiattoOrdinato.PRONTO.cambiaStato(context);
        }
    }

    @Override
    public void postPronto(StatoOrdine.OrdineContestoStato context) {
        StatoOrdine.IN_CONSEGNA.cambiaStato(context);
    }

}
