package net.lucamasini.testcase.shared.entity;

import lombok.*;
import org.jboss.errai.common.client.api.annotations.Portable;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.io.Serializable;


@NoArgsConstructor
@AllArgsConstructor
@Getter
@Setter
@ToString
@Builder(toBuilder = true)
@Portable
@Entity
@EqualsAndHashCode
@Table(name = "BAR_LAYOUT")
@NamedQueries({
        @NamedQuery(name = "BarLayout.findAll",
                query = "select l from BarLayout l order by l.bar.sigla"),
        @NamedQuery(name = "BarLayout.checkBarLayoutById",
                query = "select count(l.bar) from BarLayout l  where l.bar.id = :id"),
        @NamedQuery(name = "BarLayout.findBySigla",
                query = "select l.bar from BarLayout l  where l.bar.sigla = :sigla"),
        @NamedQuery(name = "BarLayout.findBySocieta",
                query = "select l.bar from BarLayout l where l.bar.societa = :societa"),
        @NamedQuery(name = "BarLayout.findBarLayoutWithApp",
                query = "select l from BarLayout l where l.hasApp = true order by l.bar.sigla"),
        @NamedQuery(name = "BarLayout.findBarWithoutLayout",
                query = "select b from Bar b where not b.id in (select l.id from BarLayout l) order by b.sigla"),
})
public class BarLayout implements Serializable {

    public static final long MIN_TOLLERANZA = 0;
    public static final long MIN_MAX_COMANDE_VASSOIO = 4;


    @Id
    @OneToOne
    @JoinColumn(name = "ID_BAR")
    Bar bar;

    @Version
    Long version;

    @Column(name = "MAX_COMANDE_VASSOIO")
    Long maxComandeVassoio;

    @Column(name = "FLUSSO")
    @Enumerated(EnumType.STRING)
    FlussoServizio flusso;

    @Column(name = "HAS_MONITOR_VASSOIO")
    boolean hasMonitorVassoio;

    @Column(name = "HAS_PDA_BEVANDE")
    boolean hasPdaBevande;

    @Column(name = "HAS_APP")
    boolean hasApp;

    @Column(name = "HAS_ORDINI_SOSPESI")
    boolean hasOrdiniSospesi;

    @Column(name = "TOLLERANZAINIZIOCHECKIN")
    Long tolleranzaInizioCheckIn;

    @Column(name = "TOLLERANZAFINECHECKIN")
    Long tolleranzaFineCheckIn;

    @Pattern(regexp = IPADDRESS_PATTERN, message = "Formato indirizzo ip non valido")
    @Column(name = "TERMICA_BANCONE")
    String termicaBancone;

    @Pattern(regexp = IPADDRESS_PATTERN, message = "Formato indirizzo ip non valido")
    @Column(name = "TERMICA_VASSOIO")
    String termicaVassoio;

    @NotNull
    @Pattern(regexp = "[0-2]?[0-9]:[0-9]{2}", message = "Formato orario valido mm:hh")
    @Column(name = "ORARIO_APERTURA_CUCINA")
    String orarioAperturaCucina;

    @NotNull
    @Pattern(regexp = "[0-2]?[0-9]:[0-9]{2}", message = "Formato orario valido mm:hh")
    @Column(name = "ORARIO_CHIUSURA_CUCINA")
    String orarioChiusuraCucina;

    @NotNull
    @Pattern(regexp = "[0-2]?[0-9]:[0-9]{2}", message = "Formato orario valido mm:hh")
    @Column(name = "ORARIO_APERTURA_BAR")
    String orarioAperturaBar;

    private static final String IPADDRESS_PATTERN =
            "^(([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
                    "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
                    "([01]?\\d\\d?|2[0-4]\\d|25[0-5])\\." +
                    "([01]?\\d\\d?|2[0-4]\\d|25[0-5])|)$";

    private static final String LAST_OCTECT_PATTERN = "([01]?\\d\\d?|2[0-4]\\d|25[0-5])";

    @Pattern(regexp = IPADDRESS_PATTERN, message = "Formato indirizzo ip non valido")
    @Column(name = "PYRAMID_SERVER")
    String pyramidServer;

}
