package net.lucamasini.testcase.shared.entity;

import lombok.*;
import org.jboss.errai.common.client.api.annotations.Portable;

import javax.persistence.*;


@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@ToString
@Portable
@Entity
@Table(name = "MONITOR_POSTAZIONI")
@IdClass(MonitorPostazionePK.class)
@EqualsAndHashCode(of = { "idBar", "idMonitor", "sottoPreparazione" })
@NamedQueries({
        @NamedQuery(name = "MonitorPostazione.findPostazioniByBar",
                query = "select m from MonitorPostazione m where m.idBar = :idBar"),
        @NamedQuery(name = "MonitorPostazione.findPostazioniByBarAndMonitor",
                query = "select m from MonitorPostazione m where m.idBar = :idBar and m.idMonitor = :idMonitor"),
        @NamedQuery(name = "MonitorPostazione.findPostazioniAbilitataByBarAndPostazione",
                query = "select m from MonitorPostazione m where m.idBar = :idBar and m.sottoPreparazione = :sottopreparazione and m.abilitata = true "),
})
public class MonitorPostazione {
    @Id
    @Column(name = "ID_BAR")
    String idBar;

    @Id
    @Column(name = "ID_MONITOR")
    Long idMonitor;

    @Id
    @Column(name = "SOTTOPREPARAZIONE")
    String sottoPreparazione;

    @Column(name = "ABILITATA")
    Boolean abilitata;


    /**
     * Possono essere null poiché, essendo onerosi da calcolare, il servizio chiamato potrebbe non averli impostati.
     */
    @Transient
    Long numeroPiattiInCarico;
    @Transient
    Long numeroPiattiAperti;


    public MonitorPostazione(String idBar, long idMonitor, String sottoPreparazione, boolean abilitata) {
        this.idBar = idBar;
        this.idMonitor = idMonitor;
        this.sottoPreparazione = sottoPreparazione;
        this.abilitata = abilitata;
    }
}
