package net.lucamasini.testcase.server;

import net.lucamasini.testcase.server.common.TestEm;
import net.lucamasini.testcase.server.common.producers.MockDatiUtenteProducer;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;

import javax.inject.Inject;
import javax.transaction.*;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;

import static javax.transaction.Status.STATUS_MARKED_ROLLBACK;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

public abstract class AbstractTest {

    @Inject
    protected TestEm testEm;

    @Inject
    protected MockDatiUtenteProducer datiUtenteProducer;

    @Inject
    TransactionManager tm;

    @BeforeEach
    public void init() throws SystemException, NotSupportedException {

        tm.begin();
        resetMocks();
    }

    @AfterEach
    public void after() throws HeuristicRollbackException, RollbackException, HeuristicMixedException, SystemException {

        if( tm.getStatus()==STATUS_MARKED_ROLLBACK ) {

            tm.rollback();
        } else {

            tm.commit();
        }

        testEm.deleteAll();
    }

    public void resetMocks() {
        datiUtenteProducer.setDatiUtente(null);
    }


    protected void assertWebApplicationExceptionWithStatus(Exception e, Response.Status expectedStatus) {

        assertWebApplicationExceptionWithStatus(e, expectedStatus.getStatusCode());
    }

    protected void assertWebApplicationExceptionWithStatus(Exception e, int expectedStatus) {
        //assertTrue(e instanceof EJBException);
        Throwable cause = e.getCause();
        if(e instanceof WebApplicationException) {
            cause = e;
        }

        assertTrue(cause instanceof WebApplicationException);
        WebApplicationException webApplicationException = (WebApplicationException) cause;
        assertEquals(expectedStatus, webApplicationException.getResponse().getStatus());
    }



}