package net.lucamasini.testcase.server;

import net.lucamasini.testcase.shared.dto.Canale;
import net.lucamasini.testcase.shared.dto.StatoOrdine;
import net.lucamasini.testcase.shared.dto.StatoPiattoOrdinato;
import net.lucamasini.testcase.shared.entity.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;


public class ModelHelper {
    private static long ultimoNumeroComanda;


    public static Bar createBar() {
        Bar bar = Bar.builder().filiale("2").id("U" + "-" + 491).societa("U").numero(491L).sigla("SEFB").descrizione("SESTO F.NO - BAR")
                .zona(7L).build();
        return bar;
    }

    public static BarLayout createBarLayout(Bar bar, FlussoServizio flusso) {
        BarLayout barLayout = BarLayout.builder().bar(bar).maxComandeVassoio(4L).flusso(flusso)
            .orarioChiusuraCucina("15:30").orarioAperturaCucina("11:00").orarioAperturaBar("07:30").build();
        return barLayout;
    }

    public static Ordine createOrder(Bar bar, Canale canale, StatoOrdine stato, PiattoOrdinato... piattiOrdinati) {
        long numeroComanda = ultimoNumeroComanda += 1;
        Ordine ordine = Ordine.builder().bar(bar).canale(canale).stato(stato).numeroComanda(canale.getComandaPrefix() + numeroComanda).idCatalogoOriginale(1L).build();
        ordine.setBarcode(canale.name() + stato.name() + bar.getSigla() + numeroComanda + new Date().getTime());
        final ArrayList<PiattoOrdinato> piattoOrdinatoes = new ArrayList<>();
        piattoOrdinatoes.addAll(Arrays.asList(piattiOrdinati));
        ordine.setPiattiOrdinati(piattoOrdinatoes);
        return ordine;
    }

    public static Piatto createPiatto(String nome, Piatto.Preparazione preparazione) {
        return createPiatto(nome, preparazione, null);
    }

    public static Piatto createPiatto(String nome, Piatto.Preparazione preparazione, String sottoPreparazione) {
        Piatto piatto = Piatto.builder()
                .idArticoloOriginale(1L).ean("800000001").prezzo(BigDecimal.valueOf(5))
                .nome(nome).descrizioneStampa(nome).preparazione(preparazione).sottoPreparazione(sottoPreparazione)
                .build();
        return piatto;
    }

    public static PiattoOrdinato createPiattoOrdinato(Piatto piatto, long quantita) {
        return createPiattoOrdinato(piatto, quantita, StatoPiattoOrdinato.APERTO);
    }

    public static PiattoOrdinato createPiattoOrdinato(Piatto piatto, long quantita, StatoPiattoOrdinato stato) {
        return createPiattoOrdinato(piatto, quantita, "", stato);
    }

    public static PiattoOrdinato createPiattoOrdinato(Piatto piatto, long quantita, StatoPiattoOrdinato stato, Long idMonitor) {
        return createPiattoOrdinato(piatto, quantita, "", stato, idMonitor);
    }

    public static PiattoOrdinato createPiattoOrdinato(Piatto piatto, long quantita, String nota, StatoPiattoOrdinato stato) {
        return createPiattoOrdinato(piatto, quantita, nota, stato, null);
    }

    public static PiattoOrdinato createPiattoOrdinato(Piatto piatto, long quantita, String nota, StatoPiattoOrdinato stato, Long idMonitor) {
        PiattoOrdinato piattoOrdinato = PiattoOrdinato.builder()
                .piatto(piatto).quantita(quantita).stato(stato).nota(nota).idMonitor(idMonitor).abbinamenti(new ArrayList<>())
                .build();
        for (int i = 0; i < quantita; i++) {
            piattoOrdinato.getAbbinamenti().add(createAbbinamento(piattoOrdinato, "Abbinamento " + i));
        }
        return piattoOrdinato;
    }

    public static PiattoOrdinato createPiattoOrdinato(String nome, String sottopreparazione, StatoPiattoOrdinato stato) {
        return createPiattoOrdinato(nome, sottopreparazione, stato, null);
    }

    public static PiattoOrdinato createPiattoOrdinato(String nome, String sottopreparazione, long idMonitor) {
        return createPiattoOrdinato(nome, sottopreparazione, StatoPiattoOrdinato.INCARICO, idMonitor);
    }

    public static PiattoOrdinato createPiattoOrdinato(String nome, String sottopreparazione, StatoPiattoOrdinato stato, Long idMonitor) {
        Piatto piatto = createPiatto(nome, Piatto.Preparazione.CUCINA, sottopreparazione);
        return createPiattoOrdinato(piatto, 1L, stato, idMonitor);
    }

    public static Abbinamento createAbbinamento(PiattoOrdinato piattoOrdinato, String nome) {
        Abbinamento abbinamento = Abbinamento.builder()
                .idAbbinamentoOriginale(2L).nome(nome).stato(Abbinamento.Stato.APERTO).piattoOrdinato(piattoOrdinato)
                .build();
        return abbinamento;
    }
}
