package net.lucamasini.testcase.server.boundary;

import io.quarkus.test.junit.QuarkusTest;
import net.lucamasini.testcase.server.common.BarLayoutDelegate;
import net.lucamasini.testcase.server.common.DataFactory;
import net.lucamasini.testcase.server.common.TestEm;
import net.lucamasini.testcase.server.common.producers.MockDatiUtenteProducer;
import net.lucamasini.testcase.shared.dto.DatiUtente;
import net.lucamasini.testcase.shared.dto.UserInfo;
import net.lucamasini.testcase.shared.entity.Bar;
import net.lucamasini.testcase.shared.entity.BarLayout;
import net.lucamasini.testcase.shared.entity.FlussoServizio;
import net.lucamasini.testcase.shared.security.SecurityRoles;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.inject.Inject;
import javax.ws.rs.core.Response;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;

@QuarkusTest
public class TestBarLayoutService {

    @Inject
    protected TestEm testEm;

    @Inject
    private BarLayoutDelegate delegate;

    private Map<DataFactory.SigleBar, BarLayout> layouts;
    private UserInfo userInfo;

    @Inject
    protected MockDatiUtenteProducer datiUtenteProducer;

    @BeforeEach
    public void init() {
        DataFactory.buildBars().values().forEach(b->testEm.persist(b));
        layouts= DataFactory.buildBarLayout();
        userInfo = datiUtenteProducer.utenteForBar(DataFactory.getBarLayoutWithIp(DataFactory.SigleBar.BECB), SecurityRoles.SEDE);
        datiUtenteProducer.setDatiUtente(new DatiUtente());
    }

    @AfterEach
    public void after() {
        DataFactory.buildBarLayout().values().forEach(layout->delegate.deleteLayout(layout, Response.Status.OK));
        testEm.deleteAll(Bar.class);
        datiUtenteProducer.setDatiUtente(null);

    }

    @Test
    public void testSaveModalitaNotificaDelegate(){
        Bar bar = Bar.builder().filiale("1").id("D" + "-" + 986L).societa("D").zona(97l).numero(986L).sigla("PONB").descrizione("PORTA NUOVA").build();
        BarLayout bl = BarLayout.builder().bar(bar).maxComandeVassoio(4l).flusso(FlussoServizio.BACK)
                .hasMonitorVassoio(false).hasPdaBevande(false).hasApp(true).tolleranzaInizioCheckIn(15L)
                .orarioAperturaCucina("11:00")
                .orarioChiusuraCucina("15:00")
                .orarioAperturaBar("10:00").build();
        delegate.salvaLayout(bl, Response.Status.OK).extract().as(BarLayout.class);
        bl= delegate.getLayout(bl.getBar().getId(), Response.Status.OK);
        long version = bl.getVersion();

        delegate.salvaLayout(bl, Response.Status.OK).extract().as(BarLayout.class);
        bl=delegate.getLayout(bl.getBar().getId(), Response.Status.OK);
        //TODO: capire se ha senso questa assert
        assertEquals(version,bl.getVersion());
    }
}
