package net.lucamasini.testcase.server.boundary;

import io.quarkus.test.junit.QuarkusTest;
import net.lucamasini.testcase.server.common.DataFactory;
import net.lucamasini.testcase.server.common.OrdiniResourceDelegate;
import net.lucamasini.testcase.server.common.TestEm;
import net.lucamasini.testcase.server.common.producers.MockDatiUtenteProducer;
import net.lucamasini.testcase.shared.dto.DatiUtente;
import net.lucamasini.testcase.shared.dto.UserInfo;
import net.lucamasini.testcase.shared.entity.*;
import net.lucamasini.testcase.shared.security.SecurityRoles;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static net.lucamasini.testcase.shared.dto.Canale.CHIOSCO;
import static net.lucamasini.testcase.shared.dto.StatoOrdine.APERTO;

@QuarkusTest
public class TestMergeOrdine {

    protected Bar bar;
    protected UserInfo userInfo;

    @Inject
    protected TestEm testEm;


    @Inject
    protected MockDatiUtenteProducer datiUtenteProducer;

    @Inject
    OrdiniResourceDelegate ordiniResourceDelegate;

    @BeforeEach
    public void init() {
        DataFactory.buildBars().values().forEach(b -> testEm.persist(b));
        DataFactory.buildBarLayout().values().forEach(bl -> testEm.persist(bl));
    }

    @AfterEach
    public void after() {
        testEm.deleteAll();
        datiUtenteProducer.setDatiUtente(null);
    }

    @Test
    public void testCreateAndMergeOrdine() {
        Map<DataFactory.NomiPiatti, Piatto> piatti = DataFactory.buildPiatti();

        List<BarLayoutWithIP> barLayoutsWithAppWithIP = DataFactory.buildBarLayoutWithIP().values().stream()
                .filter(barAndLayout -> barAndLayout.getBarLayout().getFlusso() == FlussoServizio.FRONT)
                .collect(Collectors.toList());

        BarLayoutWithIP barLayoutWithIP = barLayoutsWithAppWithIP.get(0);
        bar = barLayoutWithIP.getBarLayout().getBar();

        userInfo = datiUtenteProducer.utenteForBar(barLayoutWithIP, SecurityRoles.NEGOZIO);
        datiUtenteProducer.setDatiUtente(new DatiUtente());

        PiattoOrdinato panino = PiattoOrdinato.builder().piatto(piatti.get(DataFactory.NomiPiatti.PANINO)).quantita(1L).abbinamenti(new ArrayList<Abbinamento>() {{
            add(Abbinamento.builder().idAbbinamentoOriginale(1L).nome("freddo").build());
        }}).build();
        PiattoOrdinato cannelloni = PiattoOrdinato.builder().piatto(piatti.get(DataFactory.NomiPiatti.CANNELLONI)).quantita(2L).build();
        PiattoOrdinato coca = PiattoOrdinato.builder().piatto(piatti.get(DataFactory.NomiPiatti.COCA)).quantita(2L).abbinamenti(new ArrayList<Abbinamento>() {{
            add(Abbinamento.builder().idAbbinamentoOriginale(1L).nome("fredda").build());
            add(Abbinamento.builder().idAbbinamentoOriginale(1L).nome("fredda").build());
        }}).build();

        Ordine ordine = DataFactory.createOrder(bar, CHIOSCO, APERTO, panino, cannelloni, coca);
        Ordine ordineAperto = ordiniResourceDelegate.salvaOrdine(ordine);

        ordiniResourceDelegate.salvaOrdine(ordineAperto);


    }
}
