package net.lucamasini.testcase.server.common;

import net.lucamasini.testcase.server.boundary.BarLayoutWithIP;
import net.lucamasini.testcase.shared.dto.Canale;
import net.lucamasini.testcase.shared.dto.StatoOrdine;
import net.lucamasini.testcase.shared.entity.*;

import javax.enterprise.context.ApplicationScoped;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static net.lucamasini.testcase.shared.entity.FlussoServizio.BACK;
import static net.lucamasini.testcase.shared.entity.FlussoServizio.FRONT;
import static net.lucamasini.testcase.shared.entity.Piatto.Preparazione.*;

@ApplicationScoped
public class DataFactory {

    public enum NomiPiatti {
        CANNELLONI, LINGUINE, FETTINA, SALMONE, PIADINA, PANINO, FOCACCIA, CAFFE, MACEDONIA, COCA, INSALATA
    }

    public enum SigleBar {
        PONB, BECB, FAMB, SEFB, PITB
    }

    public enum NomiMenu {
        PRIMO, SECONDO, DOLCE
    }



    private static final List<Menu> menus = Arrays.asList(
            Menu.builder().uuid(UUID.randomUUID().toString()).ean("1").nome(NomiMenu.PRIMO.toString()).build(),
            Menu.builder().uuid(UUID.randomUUID().toString()).ean("12").nome(NomiMenu.SECONDO.toString()).build(),
            Menu.builder().uuid(UUID.randomUUID().toString()).ean("123").nome(NomiMenu.DOLCE.toString()).build());

    private static final List<BarLayoutWithIP> layouts = Arrays.asList(

            BarLayoutWithIP.builder().ip("127.0.0.1")
                    .barLayout(BarLayout.builder()
                            .bar(Bar.builder().filiale("1").id("D" + "-" + 986L).societa("D").numero(986L).sigla("PONB").descrizione("PORTA NUOVA").build())
                            .maxComandeVassoio(4L)
                            .flusso(FRONT)
                            .hasApp(false).hasPdaBevande(false).hasMonitorVassoio(false)
                            .tolleranzaInizioCheckIn(15L)
                            .tolleranzaFineCheckIn(60L)
                            .orarioAperturaBar("09:00")
                            .orarioAperturaCucina("11:30")
                            .orarioChiusuraCucina("14:30")
                            .build()).build(),
            BarLayoutWithIP.builder().ip("127.0.0.2")
                    .barLayout(BarLayout.builder()
                            .bar(Bar.builder().filiale("1").id("D" + "-" + 571L).societa("D").numero(571L).sigla("BECB").descrizione("BERGAMO CELADINA BAR").build())
                            .maxComandeVassoio(4L)
                            .flusso(BACK)
                            .hasApp(true).hasPdaBevande(true).hasMonitorVassoio(true)
                            .tolleranzaInizioCheckIn(15L)
                            .tolleranzaFineCheckIn(60L)
                            .orarioAperturaBar("09:00")
                            .orarioAperturaCucina("11:30").orarioChiusuraCucina("14:30")
                            .build()).build(),
            BarLayoutWithIP.builder().ip("127.0.0.3")
                    .barLayout(BarLayout.builder()
                            .bar(Bar.builder().filiale("1").id("D" + "-" + 979L).societa("D").numero(979L).sigla("FAMB").descrizione("MILANO FAMAGOSTA BAR").build())
                            .maxComandeVassoio(4L)
                            .flusso(BACK)
                            .hasApp(false).hasPdaBevande(false).hasMonitorVassoio(false)
                            .tolleranzaInizioCheckIn(15L)
                            .tolleranzaFineCheckIn(60L)
                            .orarioAperturaBar("09:00")
                            .orarioAperturaCucina("11:30").orarioChiusuraCucina("14:30")
                            .build()).build(),
            BarLayoutWithIP.builder().ip("127.0.0.4")
                    .barLayout(BarLayout.builder()
                            .bar(Bar.builder().filiale("1").id("U" + "-" + 491L).societa("U").numero(491L).sigla("SEFB").descrizione("SESTO F.NO - BAR").build())
                            .maxComandeVassoio(4L)
                            .flusso(FRONT)
                            .hasApp(false).hasPdaBevande(false).hasMonitorVassoio(false)
                            .tolleranzaInizioCheckIn(15L)
                            .tolleranzaFineCheckIn(60L)
                            .orarioAperturaBar("09:00")
                            .orarioAperturaCucina("11:30").orarioChiusuraCucina("14:30")
                            .build()).build(),
            BarLayoutWithIP.builder().ip("127.0.0.5")
                    .barLayout(BarLayout.builder()
                            .bar(Bar.builder().filiale("1").id("U" + "-" + 993L).societa("U").numero(993L).sigla("PITB").descrizione("PISTOIA BAR").build())
                            .maxComandeVassoio(4L)
                            .flusso(BACK)
                            .hasApp(true).hasPdaBevande(false).hasMonitorVassoio(true)
                            .tolleranzaInizioCheckIn(15L)
                            .tolleranzaFineCheckIn(60L)
                            .orarioAperturaBar("09:00")
                            .orarioAperturaCucina("11:30").orarioChiusuraCucina("14:30")
                            .build()).build()

    );

    private static final List<Piatto> piatti = Arrays.asList(
            Piatto.builder().idArticoloOriginale(1L).nome(NomiPiatti.CANNELLONI.name()).descrizioneStampa(NomiPiatti.CANNELLONI.name()).ean("800000001").prezzo(BigDecimal.valueOf(5)).preparazione(CUCINA).sottoPreparazione("FUOCHI").build(),
            Piatto.builder().idArticoloOriginale(2L).nome(NomiPiatti.LINGUINE.name()).descrizioneStampa(NomiPiatti.LINGUINE.name()).ean("800000001").prezzo(BigDecimal.valueOf(5)).preparazione(CUCINA).sottoPreparazione("FUOCHI").build(),
            Piatto.builder().idArticoloOriginale(3L).nome(NomiPiatti.FETTINA.name()).descrizioneStampa(NomiPiatti.FETTINA.name()).ean("800000002").prezzo(BigDecimal.valueOf(5)).preparazione(CUCINA).sottoPreparazione("GRIGLIA").build(),
            Piatto.builder().idArticoloOriginale(4L).nome(NomiPiatti.SALMONE.name()).descrizioneStampa(NomiPiatti.SALMONE.name()).ean("800000003").prezzo(BigDecimal.valueOf(5)).preparazione(CUCINA).sottoPreparazione("GRIGLIA").build(),
            Piatto.builder().idArticoloOriginale(5L).nome(NomiPiatti.PIADINA.name()).descrizioneStampa(NomiPiatti.PIADINA.name()).ean("800000004").prezzo(BigDecimal.valueOf(5)).preparazione(VASSOIO).sottoPreparazione("PIASTRA").build(),
            Piatto.builder().idArticoloOriginale(6L).nome(NomiPiatti.PANINO.name()).descrizioneStampa(NomiPiatti.PANINO.name()).ean("800000005").prezzo(BigDecimal.valueOf(5)).preparazione(VASSOIO).sottoPreparazione("PIASTRA").build(),
            Piatto.builder().idArticoloOriginale(7L).nome(NomiPiatti.FOCACCIA.name()).descrizioneStampa(NomiPiatti.FOCACCIA.name()).ean("800000006").prezzo(BigDecimal.valueOf(5)).preparazione(VASSOIO).sottoPreparazione("PIASTRA").build(),
            Piatto.builder().idArticoloOriginale(8L).nome(NomiPiatti.CAFFE.name()).descrizioneStampa(NomiPiatti.CAFFE.name()).ean("800000007").prezzo(BigDecimal.valueOf(5)).preparazione(A_VISTA).build(),
            Piatto.builder().idArticoloOriginale(9L).nome(NomiPiatti.MACEDONIA.name()).descrizioneStampa(NomiPiatti.MACEDONIA.name()).ean("800000008").prezzo(BigDecimal.valueOf(5)).preparazione(A_VISTA).build(),
            Piatto.builder().idArticoloOriginale(10L).nome(NomiPiatti.COCA.name()).descrizioneStampa(NomiPiatti.COCA.name()).ean("800000009").prezzo(BigDecimal.valueOf(5)).preparazione(A_VISTA).build(),
            Piatto.builder().idArticoloOriginale(11L).nome(NomiPiatti.INSALATA.name()).descrizioneStampa(NomiPiatti.INSALATA.name()).ean("8000000010").prezzo(BigDecimal.valueOf(5)).preparazione(VASSOIO).sottoPreparazione("INSALATE").build()
    );


    private static final List<Bar> bars = layouts.stream().map(l -> l.getBarLayout().getBar()).collect(Collectors.toList());

    public static Map<SigleBar, Bar> buildBars() {
        return bars.stream().map(bar -> bar.toBuilder().build()).collect(Collectors.toMap(b -> SigleBar.valueOf(b.getSigla()), b -> b));
    }

    public static Map<SigleBar, BarLayout> buildBarLayout() {
        return layouts.stream().map(l -> l.getBarLayout().toBuilder().build()).collect(Collectors.toMap(l -> SigleBar.valueOf(l.getBar().getSigla()), l -> l));
    }

    public static BarLayoutWithIP getBarLayoutWithIp(SigleBar sigla) {
        return buildBarLayoutWithIP().get(sigla);
    }

    public static Map<SigleBar, BarLayoutWithIP> buildBarLayoutWithIP() {
        return layouts.stream().map(l -> l.toBuilder().build()).collect(Collectors.toMap(l -> SigleBar.valueOf(l.getBarLayout().getBar().getSigla()), l -> l));
    }

    public static Map<NomiPiatti, Piatto> buildPiatti() {
        return piatti.stream().map(p -> p.toBuilder().build()).collect(Collectors.toMap(p -> NomiPiatti.valueOf(p.getNome()), p -> p));
    }

    public static Ordine createOrder(Bar bar, Canale canale, StatoOrdine stato, PiattoOrdinato... piattiOrdinati) {
        Ordine ordine = Ordine.builder().bar(bar).canale(canale).stato(stato).idCatalogoOriginale(1L).build();

        if (piattiOrdinati != null) {
            final ArrayList<PiattoOrdinato> piattoOrdinatoes = new ArrayList<>();
            piattoOrdinatoes.addAll(Arrays.asList(piattiOrdinati));
            ordine.setPiattiOrdinati(piattoOrdinatoes);
        }
        return ordine;
    }
}
