package net.lucamasini.testcase.server.common;

import io.restassured.RestAssured;
import io.restassured.http.Cookie;
import io.restassured.http.Method;
import io.restassured.response.Response;
import io.restassured.specification.RequestSpecification;
import io.smallrye.jwt.build.Jwt;
import net.lucamasini.testcase.server.restassured.RestAssuredObjectMapper;
import net.lucamasini.testcase.shared.dto.DatiUtente;
import org.eclipse.microprofile.config.inject.ConfigProperty;

import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import java.util.HashSet;

import static io.restassured.RestAssured.given;
import static io.restassured.http.ContentType.JSON;

@ApplicationScoped
public class RequestDelegate {

    @Inject
    RestAssuredObjectMapper objectMapper;

    @Inject
    DatiUtente datiUtente;

    @Inject
    @ConfigProperty(name = "smallrye.jwt.token.cookie")
    String cookieName;
    @Inject
    @ConfigProperty(name = "oauth.cookie.secure")
    boolean cookieSecure;
    @Inject
    @ConfigProperty(name = "oauth.cookie.path")
    String cookiePath;

    @PostConstruct
    void registerObjectMapper() {
        RestAssured.enableLoggingOfRequestAndResponseIfValidationFails();
        RestAssured.config().getObjectMapperConfig().defaultObjectMapper(objectMapper);
    }

    public RequestSpecification create() {
        return given()
                //.log().all()
                .cookie(generateJsonWebTokenCookie());
    }

    public <T> RequestSpecification create(T body) {
        return create()
                .body(body)
                .when()
//                .log().all()
                .contentType(JSON);
    }

    public Response getResponse(RequestSpecification specification, Method method, String path, Object... pathParams) {
        final Response response;
        switch (method) {
            case POST:
                response = specification.post(path, pathParams);
                break;
            case PUT:
                response = specification.put(path, pathParams);
                break;
            case DELETE:
                response = specification.delete(path, pathParams);
                break;
            case GET:
            default:
                response = specification.get(path, pathParams);
                break;
        }
        return response;
    }

    public Response getResponse(Method method, String path, Object... pathParams) {
        return getResponse(create(), method, path, pathParams);
    }

    private Cookie generateJsonWebTokenCookie() {
        int duration = 3600;
        String token = generateJsonWebToken(duration);
        return new Cookie.Builder(cookieName, token)
                .setPath(cookiePath).setHttpOnly(true).setSecured(cookieSecure).setMaxAge(duration)
                .build();
    }

    private String generateJsonWebToken(long duration) {
        try {
            return Jwt.claims("/jwt/jwt-claims-generic.json")
                    .expiresAt(System.currentTimeMillis() / 1000 + duration)
                    .groups(new HashSet<>(datiUtente.getUserRoles()))
                    .upn(datiUtente.getUsername())
                    .subject(datiUtente.getUsername())
                    .sign();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
