package net.lucamasini.testcase.server.common.producers;

import net.lucamasini.testcase.server.boundary.BarLayoutWithIP;
import net.lucamasini.testcase.shared.dto.DatiUtente;
import net.lucamasini.testcase.shared.dto.UserInfo;

import javax.annotation.Priority;
import javax.enterprise.context.Dependent;
import javax.enterprise.context.RequestScoped;
import javax.enterprise.inject.Alternative;
import javax.enterprise.inject.Produces;
import javax.inject.Inject;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@Dependent
@Alternative
@Priority(100)
public class MockDatiUtenteProducer {


    @Inject
    protected MockServletRequestProducer requestProducer;

    private static DatiUtente datiUtente;


    public static final String CODICE_NEGOZIO = "CODNEG";
    public static final String SOCIETA = "SOC";

    public static final String UTENTE = "TEST_USER";
    public static final String GRUPPO = "TEST_GROUP";
    public static final Integer DEF_NEGOZIO = 491;
    public static final String DEF_SOCIETA = "U";
    public static final String DEF_FILIALE = "2";


    @Produces
    @RequestScoped
    public DatiUtente getDatiUtente() {

        if (datiUtente == null) {
            datiUtente = new DatiUtente();
        }
        return datiUtente;
    }

    public void setDatiUtente(DatiUtente datiUtente) {
        this.datiUtente = datiUtente;
    }

    public UserInfo utenteForBar(BarLayoutWithIP barLayoutWithIP, String role) {
        requestProducer.setIp(barLayoutWithIP.getIp());

        UserInfo userInfo = mock(UserInfo.class);
        when(userInfo.getRoles()).thenReturn(Arrays.asList(role));
        when(userInfo.getUsername()).thenReturn(String.format("bar%sd04", barLayoutWithIP.getBarLayout().getBar().getSigla().substring(0,3).toLowerCase()));
        Map<String, String> parameters = new HashMap<>();
        parameters.put(CODICE_NEGOZIO, barLayoutWithIP.getBarLayout().getBar().getNumero().toString());
        parameters.put(SOCIETA, barLayoutWithIP.getBarLayout().getBar().getSocieta());
        when(userInfo.getUserParameters()).thenReturn(parameters);
        return userInfo;
    }

}
