package net.lucamasini.testcase.server.control;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import net.lucamasini.testcase.shared.dto.Device;
import net.lucamasini.testcase.shared.dto.StateUpdatePublisher;
import net.lucamasini.testcase.shared.entity.Ordine;
import net.lucamasini.testcase.shared.qualifiers.Publisher;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Priority;
import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.event.Event;
import javax.enterprise.inject.Alternative;
import javax.inject.Inject;
import javax.interceptor.Interceptor;
import java.util.Arrays;
import java.util.Collection;

import static net.lucamasini.testcase.shared.dto.Device.NOTIFICA_CONSEGNATO;
import static net.lucamasini.testcase.shared.dto.Device.NOTIFICA_IN_CONSEGNA;

@ApplicationScoped
@Alternative
@Publisher
@Priority(Interceptor.Priority.APPLICATION + 100)
@Slf4j
public class DummyStateUpdatePublishToCDIEvent implements StateUpdatePublisher {

    static Multimap<Ordine, Device> notified = HashMultimap.create();

    @Inject
    Event<String> notificaInConsegnaEvent;

    @Inject
    Event<String> notificaConsegnatoEvent;

    @Override
    public void publish(Ordine ordine, Device... destinations) {
        StringBuilder devices = new StringBuilder();
        for (Device device : destinations) {

            if( device == NOTIFICA_IN_CONSEGNA ) {
                notificaInConsegnaEvent.fire(ordine.getBarcode());
            }
            if( device == NOTIFICA_CONSEGNATO ) {
                notificaConsegnatoEvent.fire(ordine.getBarcode());
            }

            devices.append(device.name());
            devices.append(' ');

            notified.put(ordine, device);
        }

        log.info("Ordine {} in {}, alert a {}", ordine.getBarcode(), ordine.getStato(), devices.toString());
    }

    // Controlla che siano tutti e soli, in qualsiasi ordine
    public static boolean checkOrdineForNotifica(Ordine ordine, Device... destinations) {

        return notifichePerOrdine(ordine).containsAll(Arrays.asList(destinations)) &&
                Arrays.asList(destinations).containsAll(notifichePerOrdine(ordine));
    }

    public static Collection<Device> notifichePerOrdine(Ordine ordine) {

        return notified.get(ordine);
    }

    public static void clearNotifiche() {
        notified.clear();
    }
}
