package net.lucamasini.testcase.server.messaging;

import io.smallrye.reactive.messaging.kafka.KafkaMessage;
import org.eclipse.microprofile.config.Config;
import org.eclipse.microprofile.reactive.messaging.Message;
import org.eclipse.microprofile.reactive.messaging.spi.Connector;
import org.eclipse.microprofile.reactive.messaging.spi.IncomingConnectorFactory;
import org.eclipse.microprofile.reactive.messaging.spi.OutgoingConnectorFactory;
import org.eclipse.microprofile.reactive.streams.operators.PublisherBuilder;
import org.eclipse.microprofile.reactive.streams.operators.ReactiveStreams;
import org.eclipse.microprofile.reactive.streams.operators.SubscriberBuilder;

import javax.enterprise.context.ApplicationScoped;
import java.util.ArrayList;
import java.util.List;

import static net.lucamasini.testcase.server.messaging.DummyKafkaMessagingProvider.CONNECTOR_NAME;

@ApplicationScoped
@Connector(CONNECTOR_NAME)
public class DummyKafkaMessagingProvider implements IncomingConnectorFactory, OutgoingConnectorFactory {

  static final String CONNECTOR_NAME = "fake-kafka";

  private final List<String> list = new ArrayList<>();
  private boolean completed = false;

  @Override
  public PublisherBuilder<? extends Message<?>> getPublisherBuilder(Config config) {
    String barcode = config.getOptionalValue("barcode", String.class).orElse("BARCODE");
    String json = config.getOptionalValue("json", String.class).orElse("{}");

    return ReactiveStreams.ofNullable(KafkaMessage.of(barcode, json));
  }

  @Override
  public SubscriberBuilder<? extends Message<?>, Void> getSubscriberBuilder(Config config) {

    return ReactiveStreams.<Message<?>> builder()
            .peek(x -> list.add(x.getPayload().toString()))
            .onComplete(() -> completed = true)
            .ignore();
  }
}